function obj = Learn(obj, AlgParams, X, Y, initialWeights)

maxIters = AlgParams.maxIters;
maxIters_unc = AlgParams.maxIters_unc;
%maxFunEvals_unc = AlgParams.maxFunEvals_unc;
maxFunEvals_reg = AlgParams.maxFunEvals_reg;
%maxStepSize_unc = AlgParams.maxStepSize_unc;

lambda = AlgParams.lambda;

if nargin >= 5
  obj.W = initialWeights;
end

% Weights to be updated by GLasso are WI_regularized.
% Weights to be updated by the unconstrained step are the unregularized, nonconstant ones.
I_unc = setdiff(setdiff(1:numel(obj.W), obj.WI_constant), obj.WI_regularized);

for iter=1:maxIters
  if length(AlgParams.maxIters_unc) == maxIters
    maxIters_unc = AlgParams.maxIters_unc(iter);
  end
  
  % fminunc update for unconstrained parameters.
  [obj.W(I_unc), loss] = fminunc(@(W_unc) ComputeLoss_partial(obj, X, Y, W_unc, I_unc), obj.W(I_unc), ...
    optimset('GradObj', 'on', 'LargeScale', 'off', 'DerivativeCheck', 'off', ...
             'Display', 'iter', ...
             'MaxIter', maxIters_unc));
  disp(sprintf('fminunc loss=%f', loss));
  obj.W(obj.WI_pairwise)
  obj.W(obj.WI_obs)
  
  % GLasso update step for regularized (local) parameters.
  if ~isempty(obj.WI_regularized)
    [obj.W(obj.WI_regularized), loss] = GLassoUpdate( ...
      @(W_reg) ComputeLoss_partial(obj, X, Y, W_reg, obj.WI_regularized), ...
      @(W_reg) ComputeGradient_partial(obj, X, Y, W_reg, obj.WI_regularized), ...
      obj.W(obj.WI_regularized), lambda, maxFunEvals_reg);
  end
  disp(sprintf('GLasso(%d) loss=%f', [iter, loss]));
  
  
  % LBFGS update for unconstrained parameters.
  % TODO: Verify that we're not updating any of the local parameters.
%   W_unc_prev = obj.W(I_unc);
%   dW_unc_prev = zeros(size(I_unc));
%   for iter_unc=1:maxIters_unc
%     [W_unc, W_unc_prev, dW_unc_prev, loss] = LBFGSUpdate( ...
%       @(W_unc) ComputeLoss_partial(obj, X, Y, W_unc, I_unc), ...
%       @(W_unc) ComputeGradient_partial(obj, X, Y, W_unc, I_unc), ...
%       obj.W(I_unc), W_unc_prev, dW_unc_prev, maxStepSize_unc, maxFunEvals_unc);
%     obj.W(I_unc) = W_unc;
%     disp(sprintf('Unconstrained loss %d.%d: L=%f', [iter, iter_unc, loss]));
%   end

  % Display non-zero weights.
  I_nonzero = find(abs(obj.W(obj.WI_regularized)) >= 1e-4);
  disp([I_nonzero',obj.W(obj.WI_regularized(I_nonzero))']);
  
  % Display training error stats.
  P = Predict(obj, X);
  disp(GetConfusionStats(GetConfusion(Y == 2, P(:,2), 0.5)));
end

return

%-------------------------------------------------------------------------
function [y,g] = ComputeLoss_partial(obj, X, Y, W_partial, I_partial)
obj.W(I_partial) = W_partial;
if nargout > 1
  [y,g] = ComputeLogLoss(obj, X, Y);
  g = g(I_partial);
else
  y = ComputeLogLoss(obj, X, Y);
end
return

%-------------------------------------------------------------------------
function g_partial = ComputeGradient_partial(obj, X, Y, W_partial, I_partial)
obj.W(I_partial) = W_partial;
[y,g] = ComputeLogLoss(obj, X, Y);
g_partial = g(I_partial);
return
