function [Y, PositionMarginals, Increase, Pvalues] = Predict(obj, X)
% function Y = Predict(obj, X)
% Y is P(S|X) of size TxS.
% PositionMarginals is P(C|X) of size CxNxT
% PositionConditionals is avg_X[P(S|C,X)] of size CxNxT
% Pvalues is p-values that P(S|C,X) > P(S|X) over all X, of size CxSxN.
% Qvalues is p-values that P(C|S,X) <> P(C|X) over all X, of size CxSxN.

logAlpha = ComputeAlpha(obj, X);
logBeta = ComputeBeta(obj, X);
logP_position = ComputePositionMarginals(obj, X, logAlpha, logBeta);
logPosterior = ComputePosterior(obj, logP_position);
Y = exp(logPosterior)';

if nargout > 1
  %PositionMarginals = permute(exp(logsum(logP_position, 2)), [1 3 4 2]);
  logPositionMarginals = logsum(logP_position, 2); % Cx1xNxT
  PositionMarginals = permute(exp(logPositionMarginals), [1 3 4 2]); % CxNxT
end

if nargout > 2
  logPositionConditionals = bsxfun(@minus, ...
    logP_position, ... % CxSxNxT
    logPositionMarginals ... % Cx1xNxT
    ); % CxSxNxT
  PositionConditionals = exp(logPositionConditionals);
  Increase = bsxfun(@minus, ...
    PositionConditionals, ... % CxSxNxT
    permute(Y, [3 2 4 1]) ... % TxS -> 1xSx1xT
    ); % CxSxNxT
  %[C,S,N,T] = size(logP_position);
  
  %% This had to be looped out because ttest cannot return p of more than 2 dimensions.
  %Pvalues = zeros(C,S,N);
  %for s=1:S
  %  [Hvalues, p] = ttest(permute(Increase(:,s,:,:), [1 3 4 2]), 0, 0.05, 'right', 3);
  %  Pvalues(:,s,:) = permute(p, [1 3 2]);
  %end
  Increase = mean(Increase, 4);
end

return