function [y,g] = ComputeLogLoss(obj, X, Y, W)
% function y = ComputeLogLoss(obj, X, Y, deltaW)
%
% Computes the loss function for the given weights.

if nargin >= 4
  obj.W = W;
end

logAlpha = ComputeAlpha(obj, X);
logBeta = ComputeBeta(obj, X);
logP_position = ComputePositionMarginals(obj, X, logAlpha, logBeta);
logP_posterior = ComputePosterior(obj, logP_position);

if (nargout > 1) % Compute gradient as well.
  logP_pairwise = ComputePairwiseMarginals(obj, X, logAlpha, logBeta);
  E_empirical = ComputeEmpiricalExpectations(obj, X, Y, logP_pairwise, logP_position, logP_posterior);
  E_model = ComputeModelExpectations(obj, X, logP_pairwise, logP_position);
  g = E_model - E_empirical;
end

% Compute mean log-likelihood.
%LL = mean(log(0.5 - ((0.5 - P_posterior(1,:)') .* (3 - 2*Y)) ));
LL = 0;
for t=1:length(Y)
  LL = LL + logP_posterior(Y(t), t);
end
LL = LL / length(Y);
  
y = -LL;

return
