function [W, W_prev, dW_prev, loss] = LBFGSUpdate(fLoss, fGradient, W, W_prev, dW_prev, stepsize_max, maxFunEvals)

if (nargin < 2) || isempty(W_prev), W_prev = zeros(size(W)); end
if (nargin < 3) || isempty(dW_prev), dW_prev = zeros(size(W)); end
if (nargin < 4) || isempty(stepsize_max), stepsize_max = 10; end
if (nargin < 5) || isempty(maxFunEvals), maxFunEvals = 20; end

%
% Update weights.
%

% Compute gradient.
g = fGradient(W);
dW = -g;

%dW(obj.WI_pairwise) = 0; % Exclude obs features.

% L-BFGS.
p = W - W_prev;
v = -(dW - dW_prev);
pv = p*v';
if abs(pv) < 0.00001 % || mod(iter, NumWeights) == 0
    srcW = dW;
    disp(sprintf('Reset to gradient (|p*v|=%f)', abs(pv)));
else
    B = - p*dW'/pv;
    A = - (1 + (v*v'/pv)) * B - (v*dW'/pv);
    srcW = dW + A * p + B * v;
end
srcW = srcW ./ sqrt(sum(srcW*srcW'));

[stepsize, loss] = fminbnd(@(x) fLoss(W+x*srcW), ...
  0, stepsize_max, ...
  optimset('Display', 'off', 'MaxFunEvals', maxFunEvals));

%disp(lrate);
%dW = dW + lastdW .* momentum;
%lastdW = dW; % momentum.
W_prev = W;
dW_prev = dW;
W = W + stepsize * srcW;

return
