function [ModelParams] = alg_HHCRF_learn(AlgParams, X, Labels)
% AlgParams: struct
%   numStates: integer
%   maxIters: integer
%   report: integer; report MSE every <report> epochs (default=0-never)
% X: [NxD] training inputs in rows
% Labels: [Nx1] col vector of training labels

D = 3; % Number of obs features: 1,X,X*X.
S = 2; % Number of labels.

% Check label set.
LabelSet = sort(unique(Labels))';
if isequal(LabelSet, [0 1])
  Y = Labels + 1;
elseif isequal(LabelSet, [1 2])
  Y = Labels;
else
  error(['Labels must be 0/1 or 1/2. They are: ', mat2str(LabelSet)]);
end

% Create object.
[mu, sigma] = fitKMeans(X, AlgParams.NumStates);
ModelParams.HHCRF = HHCRF(size(X,2), AlgParams.NumStates, S, D, mu, sigma, AlgParams.initPairwise);

% Learn parameters.
if nargin >= 4
  ModelParams.HHCRF = Learn(ModelParams.HHCRF, AlgParams, X, Y, initialWeights);
else
  ModelParams.HHCRF = Learn(ModelParams.HHCRF, AlgParams, X, Y);
end

return
